/*
General purpose Spamihilator Plugin installer
by Alex Vallat

Place this file, QuerySxS.nsh, Present.ico and NotPresent.ico in a folder named "PluginInstaller"
beneath your installer script. Inlcude it as:

!include PluginInstaller\PluginInstaller.nsh

Wrap 64 bit plugin files to be installed as:
	${InstallTo64bit}
	...
	${EndInstallTo}

Wrap 32 bit plugin files to be installed as:
	${InstallTo32bit}
	...
	${EndInstallTo}

*/
SetCompressor lzma
RequestExecutionLevel admin

!include x64.nsh
!include nsDialogs.nsh
!include LogicLib.nsh
!include WinMessages.nsh
!include PluginInstaller\QuerySxS.nsh

!define NSD_SetIconSmall `!insertmacro __NSD_LoadAndSetImage file ${IMAGE_ICON} 0 "${LR_LOADFROMFILE}"`

!define SUPPORT_FILES "$INSTDIR\${PRODUCT_NAME}"
!define PRODUCT_UNINST_KEY "Software\Microsoft\Windows\CurrentVersion\Uninstall\${PRODUCT_NAME}"
!define PRODUCT_UNINST_ROOT_KEY "HKLM"
!define ICON "${NSISDIR}\Contrib\Graphics\Icons\modern-install.ico"

var boldFont
var installTargetDescription
var hasSpamihilator
var spamihilator32
var spamihilator64
var hasDotNet
var hasCrt32
var hasCrt64

Function .onInit
	InitPluginsDir
	SetOutPath $PLUGINSDIR
	File "PluginInstaller\Present.ico"
	File "PluginInstaller\NotPresent.ico"

	CreateFont $boldFont $(^Font) $(^FontSize) "700"
FunctionEnd

XPStyle on

Name "${PRODUCT_NAME} ${PRODUCT_VERSION}"
Icon "${ICON}"
UninstallIcon "${ICON}"

Page custom CreatePrequisitesDialog OnLeavePrequisites ": Missing Prerequisites"

DirText "Setup will install ${PRODUCT_NAME} for $installTargetDescription on your computer. Click Install to start the installation."

Page directory "" CustomiseDirectoryPage

Section -sys
	CreateDirectory "${SUPPORT_FILES}"

	SetOverwrite on
	WriteUninstaller "${SUPPORT_FILES}\uninstall.exe"
	SetOverwrite lastused
	WriteRegStr ${PRODUCT_UNINST_ROOT_KEY} "${PRODUCT_UNINST_KEY}" "DisplayName" "${PRODUCT_NAME}"
	WriteRegStr ${PRODUCT_UNINST_ROOT_KEY} "${PRODUCT_UNINST_KEY}" "UninstallString" "$\"${SUPPORT_FILES}\uninstall.exe$\""
	WriteRegStr ${PRODUCT_UNINST_ROOT_KEY} "${PRODUCT_UNINST_KEY}" "DisplayVersion" "${PRODUCT_VERSION}"
	WriteRegDword ${PRODUCT_UNINST_ROOT_KEY} "${PRODUCT_UNINST_KEY}" "NoModify" "1"
	WriteRegDword ${PRODUCT_UNINST_ROOT_KEY} "${PRODUCT_UNINST_KEY}" "NoRepair" "1"
	
	!ifdef PRODUCT_PUBLISHER
		WriteRegStr ${PRODUCT_UNINST_ROOT_KEY} "${PRODUCT_UNINST_KEY}" "Publisher" "${PRODUCT_PUBLISHER}"
	!endif
	
	!ifdef PRODUCT_WEB_SITE
		WriteRegStr ${PRODUCT_UNINST_ROOT_KEY} "${PRODUCT_UNINST_KEY}" "URLInfoAbout" "${PRODUCT_WEB_SITE}"
	!endif

	Call SumSize
	pop $0
    WriteRegDWORD ${PRODUCT_UNINST_ROOT_KEY} "${PRODUCT_UNINST_KEY}" "EstimatedSize" $0

SectionEnd

/*Sums the size of all sections. Return value on stack*/
Function SumSize
	/*Backup*/
	push $0
	push $1
	push $2

	StrCpy $0 0 ;size
	StrCpy $1 0 ;section idx
	loopsumsize:
	    ClearErrors
	    SectionGetFlags $1 $2
	    IfErrors endsumsize
	    IntOp $2 $2 & ${SF_SELECTED}
	    ${If} $2 <> 0
	    	SectionGetSize $1 $2
	    	IntOp $0 $0 + $2
	    ${EndIf}
	    IntOp $1 $1 + 1
	    goto loopsumsize
	endsumsize:
	
	pop $2
	pop $1
	exch $0
FunctionEnd

UninstPage uninstConfirm
UninstPage instfiles

Function un.onInit
	Call un.GetSpamihilatorLocations
	
	${If} $hasSpamihilator = 0
	    MessageBox MB_ICONSTOP|MB_OK "Uninstallation failed.$\r$\n$\r$\nCould not find Spamihilator plugins folder."
	    Abort
	${Endif}
FunctionEnd

UninstallText "This will uninstall ${PRODUCT_NAME}, including any settings. Please exit Spamihilator before continuing."

Section Uninstall
	Delete "${SUPPORT_FILES}\uninstall.exe"
	RMDir /r "${SUPPORT_FILES}"
	DeleteRegKey ${PRODUCT_UNINST_ROOT_KEY} "${PRODUCT_UNINST_KEY}"
	SetAutoClose true
	
	SetOutPath $INSTDIR
SectionEnd

Function un.onUninstSuccess
  HideWindow

  IfRebootFlag 0 noreboot
    MessageBox MB_ICONINFORMATION|MB_OK "${PRODUCT_NAME} could not be entirely removed from your computer because Spamihilator is still running. It will be removed next time Windows is restarted."
    Goto done
  noreboot:
    MessageBox MB_ICONINFORMATION|MB_OK "${PRODUCT_NAME} was successfully removed from your computer."
  done:
FunctionEnd

/*Usage: InstallFiles 64 or InstallFiles 32*/
!define InstallTo64bit "!insertmacro InstallTo 64"
!define InstallTo32bit "!insertmacro InstallTo 32"
!macro InstallTo BIT
	${IfNot} $spamihilator${BIT} == ""
		SetOutPath $spamihilator${BIT}\plugins
!macroend

!define EndInstallTo "${EndIf}"

Page instfiles

CompletedText "Completed. This plugin will be available next time Spamihilator is restarted."

Function CheckPrequisites
	#Check for .net presence
	ReadRegDword $hasDotNet HKLM "SOFTWARE\Microsoft\NET Framework Setup\NDP\v2.0.50727" "Install"
	
	#Check for CRT presence
	${WinSxS_HasAssembly} 'msvcr90.dll' 'Microsoft.VC90.CRT,type="win32",version="9.0.21022.8",processorArchitecture="x86",publicKeyToken="1fc8b3b9a1e18e3b"'
	Pop $hasCrt32
	
	#Check for Spamihilator presence
	Call GetSpamihilatorLocations

	${If} ${RunningX64}
		goto skipCrt64
	${Endif}
	    StrCmp $spamihilator64 "" skipCrt64 /*If Spamihilator64 isn't installed, then skip check for CRT64*/

		#Check for CRT 64 presence
		${WinSxS_HasAssembly} 'msvcr90.dll' 'Microsoft.VC90.CRT,type="win32",version="9.0.21022.8",processorArchitecture="amd64",publicKeyToken="1fc8b3b9a1e18e3b"'
		pop $hasCrt64
		goto endCheck64
		
    skipCrt64:
	    StrCpy $hasCrt64 1 /*Count it as having the crt64, as it doesn't need it, so isn't technically 'missing' it*/

	endCheck64:
FunctionEnd

!macro GetSpamihilatorLocations un
Function ${un}GetSpamihilatorLocations
	SetRegView 32
	ReadRegStr $spamihilator32 HKLM Software\Spamihilator InstallDir
	SetRegView lastused

	${If} ${RunningX64}
		#Check for Spamihilator64
	    SetRegView 64
	    ReadRegStr $spamihilator64 HKLM Software\Spamihilator InstallDir
	    SetRegView lastused
	${Endif}

	/*Populate the general $hasSpamihilator to be true if either 32 or 64 bit spamihilator is found, and set the install targets */
	${IfNot} $spamihilator32 == ""
		StrCpy $hasSpamihilator 1

		${IfNot} $spamihilator64 == ""
			StrCpy $INSTDIR $spamihilator64\plugins

		    StrCpy $installTargetDescription "both the x86 and x64 installations of Spamihilator"
		${Else}
			StrCpy $INSTDIR $spamihilator32\plugins

			StrCpy $installTargetDescription "Spamihilator"
		${Endif}
	${Else}
		${IfNot} $spamihilator64 == ""
			StrCpy $INSTDIR $spamihilator64\plugins

			StrCpy $installTargetDescription "Spamihilator (x64)"
			StrCpy $hasSpamihilator 1
		${Else}
			StrCpy $hasSpamihilator 0
		${Endif}
	${Endif}
FunctionEnd
!macroend

/*GetSpamihilatorLocations availabale to both installer and uninstaller*/
!insertmacro GetSpamihilatorLocations ""
!insertmacro GetSpamihilatorLocations "un."

var prqDialog

var prqTitle
var prqIconHandle
var prqDownloadSpamihilator
var prqDownloadDotNet
var prqDownloadCrt32
var prqDownloadCrt64
var prqDownloadSpamihilatorImageHandle
var prqDownloadDotNetImageHandle
var prqDownloadCrt32ImageHandle
var prqDownloadCrt64ImageHandle

/*Uses $0 and $1*/
!macro CreatePrerequisiteControls CHECK Y LABEL CONTROL ICONHANDLE
	${If} ${CHECK} = 1
		${NSD_CreateLabel} 110 ${Y}u -1 8u "${LABEL}"
		pop ${CONTROL}

		StrCpy $1 ""
	${Else}
	    StrCpy $1 "Not"
		${NSD_CreateLink} 110 ${Y}u -1 8u "${LABEL}"
		pop ${CONTROL}

		${NSD_OnClick} ${CONTROL} OnDownloadClicked
	${Endif}
	
    SendMessage ${CONTROL} ${WM_SETFONT} $boldFont 1

	IntOp $0 ${Y} - 1 /*Icon goes 1u above label*/

	${NSD_CreateIcon} 87 $0u 16 16  ""
	pop $0
	${NSD_SetIconSmall} $0 $PLUGINSDIR\$1Present.ico ${ICONHANDLE}
!macroend

!define IfAllPrerequisitesMet "!insertmacro IfAllPrerequisitesMet"
!macro IfAllPrerequisitesMet
	${If} $hasSpamihilator = 1
	${AndIf} $hasDotNet = 1
	${AndIf} $hasCrt32 = 1
	${AndIf} $hasCrt64 = 1
!macroend

Function CreatePrequisitesDialog

	Call CheckPrequisites
	
	${IfAllPrerequisitesMet}
		Abort
	${Endif}

	nsDialogs::Create /NOUNLOAD 1018
	Pop $prqDialog

	${NSD_CreateLabel} 36 0 -36 8u "Missing Prerequisites"
	Pop $prqTitle
	${NSD_CreateIcon} 0 0 32 32  ""
	Pop $0
	${NSD_SetIconFromInstaller} $0 $prqIconHandle
	${NSD_CreateLabel} 36 16u -36 24u  "Not all the prerequisites required for ${PRODUCT_NAME} are present. Please download and install these first, then run this installer again."
	Pop $0

	!insertmacro CreatePrerequisiteControls $hasSpamihilator 48 "Spamihilator" $prqDownloadSpamihilator $prqDownloadSpamihilatorImageHandle
	!insertmacro CreatePrerequisiteControls $hasDotNet 68 ".NET Framework 2.0 or later" $prqDownloadDotNet $prqDownloadDotNetImageHandle
	!insertmacro CreatePrerequisiteControls $hasCrt32 88 "Visual C++ 2008 Runtimes" $prqDownloadCrt32 $prqDownloadCrt32ImageHandle
	${IfNot} $spamihilator64 == "" /*Only require this if running spamihilator64*/
		!insertmacro CreatePrerequisiteControls $hasCrt64 108 "Visual C++ 2008 Runtimes (x64)" $prqDownloadCrt64 $prqDownloadCrt64ImageHandle
	${Endif}

	GetDlgItem $0 $HWNDPARENT 1 ; Next button
	SendMessage $0 ${WM_SETTEXT} 0 "STR:Retry"

	nsDialogs::Show

	${NSD_FreeIcon} $prqIconHandle
	${NSD_FreeImage} $prqDownloadDotNetImageHandle
	${NSD_FreeImage} $prqDownloadCrt32ImageHandle
	${NSD_FreeImage} $prqDownloadCrt64ImageHandle

FunctionEnd

Function OnDownloadClicked
	pop $0
	
	push $1 /*backup*/
	${Switch} $0
	    ${Case} $prqDownloadSpamihilator
	        StrCpy $1 "http://www.spamihilator.com/download"
	        ${Break}
		${Case} $prqDownloadDotNet
		    StrCpy $1 "http://www.microsoft.com/downloads/details.aspx?FamilyId=AB99342F-5D1A-413D-8319-81DA479AB0D7"
	        ${Break}
	    ${Case} $prqDownloadCrt32
	        StrCpy $1 "http://www.microsoft.com/downloads/details.aspx?familyid=9B2DA534-3E03-4391-8A4D-074B9F2BC1BF"
	        ${Break}
     	${Case} $prqDownloadCrt64
	        StrCpy $1 "http://www.microsoft.com/downloads/details.aspx?familyid=BD2A6171-E2D6-4230-B809-9A8D7548C1B6"
	        ${Break}
		${Default}
		    /*Unexpected click*/
		    pop $1 /*restore*/
		    return
	${EndSwitch}
	
	/*Open browser window*/
	ExecShell open $1
	
	pop $1 /*restore*/
FunctionEnd

Function OnLeavePrequisites
	/*Try detecting again*/
	Call CheckPrequisites
	
	${IfAllPrerequisitesMet}
	    Return /*Continue to next page*/
	${Else}
		SendMessage $HWNDPARENT "0x408" "0" "" /*Reload page*/
	${EndUnless}
FunctionEnd

!define HideControl "!insertmacro HideControl"
!macro HideControl ID
	GetDlgItem $1 $0 ${ID}
	ShowWindow $1 ${SW_HIDE}
!macroend
Function CustomiseDirectoryPage
	/*backup*/
	push $0
	push $1
	
	StrCpy $0 $HWNDPARENT
	${HideControl} 3 /*Back Button*/

	FindWindow $0 "#32770" "" $HWNDPARENT /*Inner dialog*/
	${HideControl} 1001 /*Browse Button*/
	${HideControl} 1008 /*Unknown checkbox*/
	${HideControl} 1019 /*Directory textbox*/
	${HideControl} 1020 /*Directory group box*/
	
	/*restore*/
	pop $1
	pop $0
FunctionEnd
