/******************************************************************************

SPAMIHILATOR - PLUGIN SDK 2.6
(c)2003-2007 Michel Krmer

plugins_services - header file

All FILES IN THIS PACKAGE ARE RELEASED UNDER THE TERMS OF THE SPAMIHILATOR
LICENSE INCLUDING THE FOLLOWING TERM THAT IS CONTRARY TO THE LICENSE:

  You are allowed to distribute verbatim copies of this SDK, but you are not
allowed to change/modify files in this package.
Futhermore you are allowed to use parts of the sample plugin's source code 
in your own Plugins as far as you make your Plugins avalibale for free and
do not charge for it or for the service of distribution!

  This file is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the Spamihilator License for more details.

  You should have received a copy of the Spamihilator license along with this
file; if not, goto www.spamihilator.com

******************************************************************************/

#ifndef __spamihilator_plugins_services_h__
#define __spamihilator_plugins_services_h__

struct SERVICE
{
	unsigned long ulSize;       //size of this structure
	char*         strReceiver;  //receiver's name for this service
	unsigned long ulService;    //service id (see below)
	short wParam;               //short parameter
	long  lParam;               //long parameter
};


typedef long (* LPSENDSERVICEFUNC)   (SERVICE *service);
typedef long (* LPRECEIVESERVICEFUNC)(SERVICE *service);


//return values
#define SERVICE_OK    0
#define SERVICE_FAIL -1


//receivers:
#define RECV_SPAMIHILATOR                  "Spamihilator"


//service IDs:

//service initialisation
#define SERV_INIT                          0x00000000

//Spamihilator wants to get the
//Receiver-Name of this plugin
#define SERV_GETRECV                       0x00000001

//Does the plugin extend the
//settings dialog?
#define SERV_GETSETTINGSEXT                0x00000002

//Spamihilator asks if the plugin
//wants the highest filter priority
//this is useful for white-lists and so on...
#define SERV_WANTHIGHESTPRIORITY           0x00000003

//The language has changed
#define SERV_LANGUAGECHANGED               0x00000004

//Spamihilator is learning
#define SERV_LEARN                         0x00000005

//Spamihilator asks if the plugin
//wants to decide which mails should
//be saved in the Training Area
#define SERV_WANTTRAININGAREACONTROL       0x00000006

//let the plugin decide!
#define SERV_TRAININGAREACONTROL           0x00000007


//the plugin wants Spamihilator to
//translate a window
#define SERV_TRANSLATEWINDOW               0x10000000

//the plugin wants Spamihilator to
//translate a menu
#define SERV_TRANSLATEMENU                 0x10000001

//the plugin wants Spamihilator to
//translate a string
#define SERV_TRANSLATESTRING               0x10000002

//tell Spamihilator the spam words
//that have been found
#define SERV_FOUNDSPAMWORDS                0x10000003

//get Spamihilator version
#define SERV_GETSPAMIHILATORVER            0x10000004

//get the plain text of an e-mail message
#define SERV_GETPLAINTEXT                  0x10000005

//mime encoding
#define SERV_MIMEENCODE                    0x10000006

//mime decoding
#define SERV_MIMEDECODE                    0x10000007

//set a configuration value
#define SERV_SETCONFIGVALUE                0x10000008

//get a configuration value
#define SERV_GETCONFIGVALUE                0x10000009

//let Spamihilator learn!
#define SERV_LEARNFROMMESSAGE              0x10000010

//don't show this message in the
//Training Area
#define SERV_NOTRAINING                    0x10000011

//Spamihilator has to show a short
//text in the Status Window
#define SERV_SHOWSTATUSTEXT                0x10000012

//get the configuration path
//(the path where spamihilator.ini resides)
#define SERV_GETCONFIGPATH                 0x10000013


//all user-defined services must
//have at least this value
#define SERV_USER                          0xA0000000


//SERV_WANTTRAININGAREACONTROL return values
#define SERVWANTTRAININGAREACONTROL_HEADERS     01
#define SERVWANTTRAININGAREACONTROL_MAILBODY    02
#define SERVWANTTRAININGAREACONTROL_FULLMESSAGE 04

//SERV_TRAININGAREACONTROL return values
#define SERVTRAININGAREACONTROL_SHOWMESSAGE         0x0000
#define SERVTRAININGAREACONTROL_HIDEMESSAGE         0x0010
#define SERVTRAININGAREACONTROL_LEARN               0x0100



//SERV_GETSETTINGSEXT parameter
struct SERVGETSETTINGSEXTITEM
{
	unsigned long ulSize;       //size of this structure
	HINSTANCE     hInstance;    //handle to instance containing the resources
	unsigned int  uID;          //identifier of a dialog box
	DLGPROC       dlgProc;      //DialogProc for this dialog box
	char*         strTitle;     //item title
	unsigned int  uParentID;    //identifier of parent node (dialog box identifier)
	bool          bChildren;    //does this node have children?
	HANDLE        hImage;       //icon handle
	char*         strHelpFile;  //help filename
};

struct SERVGETSETTINGSEXT
{
	unsigned long nItems;       //number of items
	SERVGETSETTINGSEXTITEM items[1]; //pointer to the first item
};

#define WM_APPLYCHANGES WM_USER + 1001   //will be sent to all settings dialogs


//SERV_TRANSLATESTRING parameter
struct SERVTRANSSTR
{
	unsigned long ulSize;       //size of this structure
	char*         strSrc;       //source string
	char*         strDest;      //destination string
	int           nDestMax;     //maximum destination string length
};


//SERV_LEARN parameter
#define SERVLEARNGOOD  0
#define SERVLEARNBAD   1

#define SERVLEARNNONE -1


//SERV_GETPLAINTEXT parameter
struct SERVGETPLAINTEXT
{
	unsigned long ulSize;       //size of this structure
	const char*   strSrc;       //source string ('\0' terminated!)
	char*         strDest;      //destination string
	int           nDestMax;     //maximum destination string length
	BOOL          bStripHTML;   //to strip HTML or not - that's the question...
};


//SERV_MIMEENCODE, SERV_MIMEDECODE parameters
#define SERVMIMEBASE64          1
#define SERVMIMEQUOTEDPRINTABLE 2
#define SERVMIMEHEADER          4

struct SERVMIME
{
	unsigned long ulSize;       //size of this structure
	const char*   strSrc;       //source string
	char*         strDest;      //destination string
	int           nDestMax;     //maximum destination string length
	short         flags;        //MIME flags (e.g. SERVMIMEBASE64)
};


//SERV_SETCONFIGVALUE, SERV_GETCONFIGVALUE parameter

struct SERVCONFIGVALUE
{
	unsigned long ulSize;       //size of this structure
	HINSTANCE     hInstance;    //the plugin's module handle
	const char*   strKey;       //key name
	char*         strValue;     //value string
	int           nDestMax;     //maximum destination string length (only used for SERV_GETCONFIGVALUE)
};

#endif
